<?php
/* Copyright (C) 2003      Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2004-2005 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2004      Sebastien Di Cintio  <sdicintio@ressource-toi.org>
 * Copyright (C) 2004      Benoit Mortier       <benoit.mortier@opensides.be>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: modComptabilite.class.php,v 1.37 2005/10/22 13:45:24 eldy Exp $
 * $Source: /cvsroot/dolibarr/dolibarr/htdocs/includes/modules/modComptabilite.class.php,v $
 *
 */

/**     \defgroup   comptabilite     Module comptabilite
        \brief      Module pour inclure des fonctions de comptabilit (gestion de comptes comptables et rapports)
*/

/**
        \file       htdocs/includes/modules/modComptabilite.class.php
        \ingroup    comptabilite
        \brief      Fichier de description et activation du module Comptabilite
*/

include_once "DolibarrModules.class.php";

/** \class modComptabilite
        \brief      Classe de description et activation du module Comptabilite
*/

class modComptabilite extends DolibarrModules
{

   /**
    *   \brief      Constructeur. Definit les noms, constantes et boites
    *   \param      DB      handler d'accs base
    */
  function modComptabilite($DB)
  {
    global $conf;

    $this->db = $DB ;
    $this->numero = 10 ;
    
    $this->family = "financial";
    $this->name = "Comptabilite";
    $this->description = "Gestion sommaire de comptabilit";

    $this->revision = explode(" ","$Revision: 1.37 $");
    $this->version = $this->revision[1];

    $this->const_name = 'MAIN_MODULE_COMPTABILITE';

    // Config pages
    $this->config_page_url = "compta.php";

    // Dpendances
    $this->requiredby = array("modFacture");
	$this->depends = array("modFacture","modBanque","modProduit","modSociete","modUser");
    $this->conflictwith = array("modComptabiliteExpert");

    // Constantes
    $this->const = array();

    // Rpertoires
    $this->dirs = array();
    $this->dirs[0] = $conf->compta->dir_output;
    $this->dirs[1] = $conf->compta->dir_output."/rapport";
    $this->dirs[2] = $conf->compta->dir_output."/export";
    $this->dirs[3] = $conf->compta->dir_images;

    // Boites
    $this->boxes = array();

    // Permissions
    $this->rights = array();
    $this->rights_class = 'compta';
    $r=0;
    
    $r++;
    $this->rights[$r][0] = 91;
    $this->rights[$r][1] = 'Lire les charges';
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 1;
    $this->rights[$r][4] = 'charges';
    $this->rights[$r][5] = 'lire';

    $r++;
    $this->rights[$r][0] = 92;
    $this->rights[$r][1] = 'Crer modifier les charges';
    $this->rights[$r][2] = 'w';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'charges';
    $this->rights[$r][5] = 'creer';

    $r++;
    $this->rights[$r][0] = 93;
    $this->rights[$r][1] = 'Supprimer les charges';
    $this->rights[$r][2] = 'd';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'charges';
    $this->rights[$r][5] = 'supprimer';

    $r++;
    $this->rights[$r][0] = 95;
    $this->rights[$r][1] = 'Lire CA, bilans, rsultats';
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 1;
    $this->rights[$r][4] = 'resultat';
    $this->rights[$r][5] = 'lire';

    $r++;
    $this->rights[$r][0] = 96;
    $this->rights[$r][1] = 'Paramtrer la ventilation';
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'ventilation';
    $this->rights[$r][5] = 'parametrer';

    $r++;
    $this->rights[$r][0] = 97;
    $this->rights[$r][1] = 'Ventiler les lignes de facture';
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'ventilation';
    $this->rights[$r][5] = 'creer';

    $r++;
    $this->rights[$r][0] = 98;
    $this->rights[$r][1] = "Accs  l'espace compta/trso";
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'general';
    $this->rights[$r][5] = 'lire';

    $r++;
    $this->rights[$r][0] = 10000;
    $this->rights[$r][1] = 'Visualiser les dpenses';
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'depense';
    $this->rights[$r][5] = 'lire';

    $r++;
    $this->rights[$r][0] = 10010;
    $this->rights[$r][1] = 'Saisir des dpenses';
    $this->rights[$r][2] = 'w';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'depense';
    $this->rights[$r][5] = 'creer';

    $r++;
    $this->rights[$r][0] = 10100;
    $this->rights[$r][1] = 'Visualiser les recettes';
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'recette';
    $this->rights[$r][5] = 'lire';

    $r++;
    $this->rights[$r][0] = 10110;
    $this->rights[$r][1] = 'Saisir des recettes';
    $this->rights[$r][2] = 'w';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'recette';
    $this->rights[$r][5] = 'creer';
	
    $r++;
    $this->rights[$r][0] = 10200;
    $this->rights[$r][1] = 'Visualiser les oprations';
    $this->rights[$r][2] = 'r';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'operation';
    $this->rights[$r][5] = 'lire';

    $r++;
    $this->rights[$r][0] = 10210;
    $this->rights[$r][1] = 'Saisir des oprations';
    $this->rights[$r][2] = 'w';
    $this->rights[$r][3] = 0;
    $this->rights[$r][4] = 'operation';
    $this->rights[$r][5] = 'creer';
  }


   /**
    *   \brief      Fonction appele lors de l'activation du module. Insre en base les constantes, boites, permissions du module.
    *               Dfinit galement les rpertoires de donnes  crer pour ce module.
    */
  function init()
  {
    global $conf;
    
    // Nettoyage avant activation
    $this->remove();

    return $this->_init($sql);
  }

  /**
   *    \brief      Fonction appele lors de la dsactivation d'un module.
   *                Supprime de la base les constantes, boites et permissions du module.
   */
  function remove()
  {
    $sql = array();

    return $this->_remove($sql);
  }
}
?>
